var React = require('react');
var utils = require('styletron-utils');

var isValidAttr = require('./is-valid-attr');

var STYLES_KEY = '__STYLETRON_STYLES';
var TAG_KEY = '__STYLETRON_TAG';

module.exports = applyStyle;

/**
 * Higher-order component for presentational components
 * @packagename styletron-react
 * @param  {function} styleFn Styler function
 * @return {function}         Higher order component
 * @example
 * class Panel extends React.Component {
 *   render() {
 *     <div className={this.styles.container}>
 *       <h1 className{this.styles.title}>Panel</h1>
 *       {this.props.children}
 *     </div>
 *   }
 * }
 *
 * const hoc = connectToStyles(props => {
 *   return {
 *     container: {
 *       width: '640px',
 *       background: props.color
 *     },
 *     title: {
 *       fontSize: '40px'
 *     }
 *   };
 * });
 *
 * module.exports = hoc(Panel);
 */
function applyStyle(base, styleArg) {
  if (typeof base === 'string') {
    // Element
    return createComponent(base, [styleArg]);
  }
  if (typeof base === 'function' && base[TAG_KEY] && base[STYLES_KEY]) {
    // Component
    return createComponent(base[TAG_KEY], base[STYLES_KEY].concat(styleArg));
  }
  throw Error('Must pass in element or component');
}

function createComponent(tagName, stylesArray) {
  var StyledElement = function (props, context) {
    
    var resolvedStyle = {};
    StyledElement[STYLES_KEY].forEach(function (style) {
      if (typeof style === 'function') {
        assign(resolvedStyle, style(props, context));
      } else if (typeof style === 'object') {
        assign(resolvedStyle, style);
      }
    });

    var styletronClassName = utils.injectStylePrefixed(context.styletron, resolvedStyle);

    var elementProps = omitInvalidProps(props);
    elementProps.className = props.className
      ? ((props.className) + " " + styletronClassName)
      : styletronClassName;

    return React.createElement(StyledElement[TAG_KEY], elementProps);
  };

  StyledElement[TAG_KEY] = tagName;
  StyledElement[STYLES_KEY] = stylesArray;
  StyledElement.contextTypes = {styletron: React.PropTypes.object};

  return StyledElement;
}

function assign(target, source) {
  for (key in source) {
    target[key] = source[key];
  }
}

function omitInvalidProps(props) {
  var target = {};
  for (var attr in props) {
    if (isValidAttr(attr)) {
      target[attr] = props[attr];
    }
  }
  return target;
}
