var React = require('react');
var utils = require('styletron-utils');

module.exports = connectToStyles;

/**
 * Higher-order component for presentational components
 * @packagename styletron-react
 * @param  {function} styleFn Styler function
 * @return {function}         Higher order component
 * @example
 * class Panel extends React.Component {
 *   render() {
 *     <div className={this.styles.container}>
 *       <h1 className{this.styles.title}>Panel</h1>
 *       {this.props.children}
 *     </div>
 *   }
 * }
 *
 * const hoc = connectToStyles(props => {
 *   return {
 *     container: {
 *       width: '640px',
 *       background: props.color
 *     },
 *     title: {
 *       fontSize: '40px'
 *     }
 *   };
 * });
 *
 * module.exports = hoc(Panel);
 */
function connectToStyles(styleFn) {
  return function hoc(Component) {
    var Connected = (function (superclass) {
      function Connected () {
        superclass.apply(this, arguments);
      }

      if ( superclass ) Connected.__proto__ = superclass;
      Connected.prototype = Object.create( superclass && superclass.prototype );
      Connected.prototype.constructor = Connected;

      Connected.prototype.render = function render () {
        var this$1 = this;

        var styles = styleFn(this.props);
        var classMap = Object.keys(styles).reduce(function (acc, key) {
          acc[key] = utils.injectStyle(this$1.context.styletron, styles[key])
          return acc;
        }, {});
        return React.createElement(Component, Object.assign({
          styles: classMap,
          ref: function (c) {this$1.connectedElement = c}
        }, this.props));
      };

      return Connected;
    }(React.Component));
    Connected.contextTypes = {styletron: React.PropTypes.object.isRequired};
    Connected.displayName = "connectToStyles(" + (getDisplayName(Component)) + ")";
    if (Component.defaultProps) {
      Connected.defaultProps = Component.defaultProps;
    } else if (typeof Component.getDefaultProps === 'function') {
      Connected.defaultProps = Component.getDefaultProps();
    }
    return Connected;
  };
}

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}
